import os
import sys
import argparse
import json
from glob import glob
import mimetypes as mt

import itkdb

def upload(client, input_name, project, upload_file = True):
        
    # Read test data from json
    with open (input_name) as f:
        data = json.load(f)
    
    # Get component
    comp = client.get('getComponent', json={"component" : data["component"], "alternativeIdentifier" : data.get("alternativeIdentifier", True)})
    
    # Check stage
    if comp["currentStage"]["code"] != data["currentStage"]:
        print("WARNING: Component stage in DB {} doesn't match that requested {}.  Skipping".format(comp["currentStage"]["code"], data["currentStage"]))
        return None

    # Check type
    if comp["componentType"]["code"] != data["componentType"]:  
        print("WARNING: Component type in DB {} doesn't match that requested {}.  Skipping".format(comp["componentType"]["code"], data["componentType"]))
        return None
    
    # Get test template 
    test_template = client.get('generateTestTypeDtoSample', json={"project" : project, 
                                                                  "componentType" : data["componentType"], 
                                                                  'code': data["testType"]}) #, 'requiredOnly' : True})

   
     
        
    # Uploate test result
    new_test_result = client.post('uploadTestRunResults', json={**test_template, 'component': comp["code"], 
                                                                "institution" : data["institution"],
                                                                "passed" : data["passed"], 
                                                                "problems" : data["problems"], 
                                                                "date" : data["date"],
                                                                "properties" : data["properties"], 
                                                                "results" : data.get("results", {}), 
                                                                'runNumber': str(data["runNumber"]),
                                                                "comments" : data.get("comments", []), 
                                                                "defects" : data.get("defects", [])
                                                               })
    
    
    
    
    # Upload file 
    if "attachments" in data:
        for file in data["attachments"]:
            client.post('createTestRunAttachment', data={
                "testRun": new_test_result['testRun']['id'],
                "type": "file",
                "title": file["title"],
                "description": file["description"] },
                files={"data": (os.path.basename(file["filename"]), open(file["filename"], "rb"), mt.guess_type(file["filename"])[0])
                })
            
    else:
        print("No attachments to upload")
    
    
    
    # Print results link
    host = "https://itkpd-test.unicorncollege.cz/"
    #link = "{}{}/testRunView?id={}".format(host, new_test_result["componentTestRun"]["awid"], new_test_result['testRun']['id'])
    link = "{}/testRunView?id={}".format(host, new_test_result['testRun']['id'])
    print("{}\t{}".format(comp["code"], link))
    
    return new_test_result





if __name__ == "__main__":

    # Parse command line
    parser = argparse.ArgumentParser(description="Script for uploading strip hyrbid test results")
    parser.add_argument("json", help="Full name of json file containing test results or directory of json files")
    parser.add_argument("--accessCode1", default = None, help = "Access Code 1 (can also be specified via ITKDB_ACCESS_CODE1 env var)")
    parser.add_argument("--accessCode2", default = None, help = "Access Code 2 (can also be specified via ITKDB_ACCESS_CODE2 env var)")
    parser.add_argument("--project", "-p", default = "S", help = "Project code (S/P)")
    args = parser.parse_args()


    # Authenticate
    if args.accessCode1 and args.accessCode2:
        u = itkdb.core.User(access_code1=args.accessCode1, access_code2=args.accessCode2)
        client = itkdb.Client(user = u)
    elif "ITKDB_ACCESS_CODE1" in os.environ and "ITKDB_ACCESS_CODE2" in os.environ:
        client = itkdb.Client()
    else:
        sys.exit("ERROR: Must provide access codes, either via arguments or environment variables")

    client.user.authenticate()
    user = client.get('getUser', json={'userIdentity': client.user.identity})
    print ('>>> Hello {} {}, welcome to the strip hybrid upload script'.format(user["firstName"], user["lastName"]))

    # Attempt to upload test result
    if args.json.endswith(".json"):
        print(f">>> Uploading {args.json} ...")
        upload(client, args.json, args.project)
    elif os.path.isdir(args.json):
        files = glob(f"{args.json}/*.json")
        if not files:
            sys.exit(f"ERROR: No json files found in dir {args.json}")
        for f in files:
            print(f">>> Uploading {f} ...")
            upload(client, f, args.project)
    else:
        sys.exit("ERROR: Unknown input format.  Plese provide a json file or directory path containing json files")
            
        
    
